/*
 * Copyright (c) 2011-2012 Xsigo Systems. All rights reserved
 *
 * This software is available to you under a choice of one of two
 * licenses.  You may choose to be licensed under the terms of the GNU
 * General Public License (GPL) Version 2, available from the file
 * COPYING in the main directory of this source tree, or the
 * OpenIB.org BSD license below:
 *
 *     Redistribution and use in source and binary forms, with or
 *     without modification, are permitted provided that the following
 *     conditions are met:
 *
 *      - Redistributions of source code must retain the above
 *        copyright notice, this list of conditions and the following
 *        disclaimer.
 *
 *      - Redistributions in binary form must reproduce the above
 *        copyright notice, this list of conditions and the following
 *        disclaimer in the documentation and/or other materials
 *        provided with the distribution.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS
 * BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN
 * ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
 * CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

#include "xve.h"
#include "xve_compat.h"

static int xve_max_inline_data = 128;
module_param(xve_max_inline_data, int, 0644);

static int xve_use_hugecq = 16384;
module_param(xve_use_hugecq, int, 0644);

int xve_mcast_attach(struct net_device *dev, u16 mlid, union ib_gid *mgid,
		     int set_qkey)
{
	struct xve_dev_priv *priv = netdev_priv(dev);
	struct ib_qp_attr *qp_attr = NULL;
	int ret;
	u16 pkey_index;

	if (ib_find_pkey(priv->ca, priv->port, priv->pkey, &pkey_index)) {
		clear_bit(XVE_PKEY_ASSIGNED, &priv->flags);
		ret = -ENXIO;
		goto out;
	}
	set_bit(XVE_PKEY_ASSIGNED, &priv->flags);

	if (set_qkey) {
		ret = -ENOMEM;
		qp_attr = kmalloc(sizeof(*qp_attr), GFP_KERNEL);
		if (!qp_attr)
			goto out;

		/* set correct QKey for QP */
		qp_attr->qkey = priv->qkey;
		ret = ib_modify_qp(priv->qp, qp_attr, IB_QP_QKEY);
		if (ret) {
			xve_warn(priv, "failed to modify QP, ret = %d\n", ret);
			goto out;
		}
	}

	/* attach QP to multicast group */
	ret = ib_attach_mcast(priv->qp, mgid, mlid);
	if (ret)
		xve_warn(priv,
			"failed to attach to multicast group, ret = %d\n",
			ret);
	else
		priv->counters[XVE_MCAST_ATTACH]++;

	xve_dbg_mcast(priv, "Successfully attached to Group mode%d", set_qkey);
out:
	kfree(qp_attr);
	qp_attr = NULL;
	return ret;
}

int xve_init_qp(struct net_device *dev)
{
	struct xve_dev_priv *priv = netdev_priv(dev);
	int ret;
	struct ib_qp_attr qp_attr;
	int attr_mask;

	if (!test_bit(XVE_PKEY_ASSIGNED, &priv->flags)) {
		xve_warn(priv, "PKEY not assigned\n");
		return -1;
	}

	qp_attr.qp_state = IB_QPS_INIT;
	qp_attr.qkey = 0;
	if (priv->is_eoib) {
		qp_attr.qkey =  priv->gw.t_qkey;
		xve_debug(DEBUG_QP_INFO, priv, "Setting qkey %x\n",
			qp_attr.qkey);
	}
	qp_attr.port_num = priv->port;
	qp_attr.pkey_index = priv->pkey_index;
	attr_mask = IB_QP_QKEY | IB_QP_PORT | IB_QP_PKEY_INDEX | IB_QP_STATE;
	ret = ib_modify_qp(priv->qp, &qp_attr, attr_mask);
	if (ret) {
		xve_warn(priv, "failed to modify QP to init, ret = %d\n", ret);
		goto out_fail;
	}

	qp_attr.qp_state = IB_QPS_RTR;
	/* Can't set this in a INIT->RTR transition */
	attr_mask &= ~IB_QP_PORT;
	ret = ib_modify_qp(priv->qp, &qp_attr, attr_mask);
	if (ret) {
		xve_warn(priv, "failed to modify QP to RTR, ret = %d\n", ret);
		goto out_fail;
	}

	qp_attr.qp_state = IB_QPS_RTS;
	qp_attr.sq_psn = 0;
	attr_mask |= IB_QP_SQ_PSN;
	attr_mask &= ~IB_QP_PKEY_INDEX;
	ret = ib_modify_qp(priv->qp, &qp_attr, attr_mask);
	if (ret) {
		xve_warn(priv, "failed to modify QP to RTS, ret = %d\n", ret);
		goto out_fail;
	}

	return 0;

out_fail:
	qp_attr.qp_state = IB_QPS_RESET;
	if (ib_modify_qp(priv->qp, &qp_attr, IB_QP_STATE))
		xve_warn(priv, "Failed to modify QP to RESET state\n");

	return ret;
}

int xve_transport_dev_init(struct net_device *dev, struct ib_device *ca)
{
	struct xve_dev_priv *priv = netdev_priv(dev);
	struct ib_cq_init_attr cq_attr = {};
	struct ib_qp_init_attr init_attr = {
		.cap = {
			.max_send_wr = priv->xve_sendq_size,
			.max_recv_wr = priv->xve_recvq_size,
			.max_send_sge = 1,
			.max_recv_sge = xve_ud_rx_sg(priv)},
		.sq_sig_type = IB_SIGNAL_ALL_WR,
		.qp_type = IB_QPT_UD
	};
	struct ethtool_coalesce *coal;
	int ret, size = 0, max_sge = MAX_SKB_FRAGS + 1,req_vec;
	int i;

	priv->ca->attrs.device_cap_flags &= IB_DEVICE_LOCAL_DMA_LKEY;
	priv->pd = ib_alloc_pd(priv->ca, 0);
	if (IS_ERR(priv->pd)) {
		pr_warn("%s: failed to allocate PD for %s\n",
				ca->name, priv->xve_name);
		return -ENODEV;
	}


	ret = xve_cm_dev_init(dev);
	if (ret != 0) {
		pr_err("%s Failed for %s [ret %d ]\n", __func__,
				priv->xve_name, ret);
		goto out_free_mr;
	}

	/* Bug 24673784 */
	if (priv->is_titan && xve_use_hugecq)
		priv->xve_rcq_size = priv->xve_scq_size =
				xve_use_hugecq;
	else {
		size = priv->xve_sendq_size;
		size += priv->xve_recvq_size + 1; /* 1 extra for rx_drain_qp */
		priv->xve_rcq_size = size;
		priv->xve_scq_size = priv->xve_sendq_size;
	}

	req_vec = (priv->port - 1) * 2;

	/* Create Receive CompletionQueue */
	cq_attr.cqe = priv->xve_rcq_size;
	cq_attr.comp_vector = (req_vec) % priv->ca->num_comp_vectors;
	priv->recv_cq = ib_create_cq(priv->ca, xve_ib_rx_completion, NULL,
				     priv, &cq_attr);
	if (IS_ERR(priv->recv_cq)) {
		pr_warn("%s: failed to create receive CQ for %s size%d\n",
			ca->name, priv->xve_name, priv->xve_rcq_size);
		goto out_free_mr;
	}

	/* Create Send CompletionQueue */
	cq_attr.cqe = priv->xve_scq_size;
	cq_attr.comp_vector = (req_vec + 1) % priv->ca->num_comp_vectors;
	priv->send_cq = ib_create_cq(priv->ca, xve_ib_tx_completion, NULL,
				     priv, &cq_attr);
	if (IS_ERR(priv->send_cq)) {
		pr_warn("%s: failed to create send CQ for %s size%d\n",
			ca->name, priv->xve_name, priv->xve_scq_size);
		goto out_free_recv_cq;
	}

	if (ib_req_notify_cq(priv->recv_cq, IB_CQ_NEXT_COMP))
		goto out_free_send_cq;
	if (ib_req_notify_cq(priv->send_cq, IB_CQ_NEXT_COMP))
		goto out_free_send_cq;

	coal = kzalloc(sizeof(*coal), GFP_KERNEL);
	if (coal) {
		coal->rx_coalesce_usecs = 10;
		coal->tx_coalesce_usecs = 10;
		coal->rx_max_coalesced_frames = 16;
		coal->tx_max_coalesced_frames = 16;
		dev->ethtool_ops->set_coalesce(dev, coal);
		kfree(coal);
		coal = NULL;
	}

	init_attr.send_cq = priv->send_cq;
	init_attr.recv_cq = priv->recv_cq;

	if (priv->hca_caps & IB_DEVICE_MANAGED_FLOW_STEERING)
		init_attr.create_flags |= IB_QP_CREATE_NETIF_QP;

	if (priv->hca_caps & IB_DEVICE_BLOCK_MULTICAST_LOOPBACK)
		init_attr.create_flags |= IB_QP_CREATE_BLOCK_MULTICAST_LOOPBACK;

	if (priv->is_eoib && priv->is_titan) {
		pr_warn("%s: failed no create eoib flag \n", ca->name);
		init_attr.cap.max_inline_data = xve_max_inline_data;
	}

	if (dev->features & NETIF_F_SG) {
		/*PSIF supports 16 SGE's*/
		max_sge =  min_t(uint32_t, priv->dev_attr.max_sge_rd,
				MAX_SKB_FRAGS + 1);

		init_attr.cap.max_send_sge = max_sge;
	}

	xve_debug(DEBUG_QP_INFO, priv, "Create QP flags%x sge%d\n",
				init_attr.create_flags, max_sge);

	priv->qp = ib_create_qp(priv->pd, &init_attr);
	if (IS_ERR(priv->qp)) {
		pr_warn("%s: failed to create QP\n", ca->name);
		goto out_free_send_cq;
	}

	for (i = 0; i < max_sge; ++i)
		priv->tx_sge[i].lkey = priv->pd->local_dma_lkey;

	priv->tx_wr.wr.opcode = IB_WR_SEND;
	priv->tx_wr.wr.sg_list = priv->tx_sge;
	priv->tx_wr.wr.send_flags = IB_SEND_SIGNALED;

	priv->rx_sge[0].lkey = priv->pd->local_dma_lkey;
	 if (xve_ud_need_sg(priv->admin_mtu)) {
		priv->rx_sge[0].length = XVE_UD_HEAD_SIZE;
		for (i = 1; i < xve_ud_rx_sg(priv); i++) {
			priv->rx_sge[i].length = PAGE_SIZE;
			priv->rx_sge[i].lkey = priv->pd->local_dma_lkey;
		}
		priv->rx_wr.num_sge = xve_ud_rx_sg(priv);
	} else {
		priv->rx_sge[0].length = XVE_UD_BUF_SIZE(priv->max_ib_mtu);
		priv->rx_wr.num_sge = 1;
	}
	priv->rx_wr.next = NULL;
	priv->rx_wr.sg_list = priv->rx_sge;
	priv->max_send_sge = init_attr.cap.max_send_sge;

	return 0;

out_free_send_cq:
	ib_destroy_cq(priv->send_cq);
	priv->send_cq = NULL;

out_free_recv_cq:
	ib_destroy_cq(priv->recv_cq);
	priv->recv_cq = NULL;

out_free_mr:

	xve_cm_dev_cleanup(dev);

out_free_pd:
	ib_dealloc_pd(priv->pd);
	return -ENODEV;
}

void xve_transport_dev_cleanup(struct net_device *dev)
{
	struct xve_dev_priv *priv = netdev_priv(dev);
	int ret;

	/* Destroy QP */
	if (priv->qp) {
		ret = ib_destroy_qp(priv->qp);
		if (ret)
			xve_warn(priv,
				"ib_qp_destroy failed (ret = %d)\n", ret);

		priv->qp = NULL;
		clear_bit(XVE_PKEY_ASSIGNED, &priv->flags);
	}

	ib_destroy_cq(priv->send_cq);

	ib_destroy_cq(priv->recv_cq);

	xve_cm_dev_cleanup(dev);

	ib_dealloc_pd(priv->pd);

}

void xve_event(struct ib_event_handler *handler, struct ib_event *record)
{
	struct xve_dev_priv *priv =
	    container_of(handler, struct xve_dev_priv, event_handler);

	if (record->element.port_num != priv->port)
		return;

	xve_debug(DEBUG_MCAST_INFO, priv, "Event %d on device %s port %d\n",
		  record->event, record->device->name,
		  record->element.port_num);

	switch (record->event) {
	case IB_EVENT_SM_CHANGE:
			priv->counters[XVE_SM_CHANGE_COUNTER]++;
			xve_queue_work(priv, XVE_WQ_START_FLUSHLIGHT);
			break;
	case IB_EVENT_CLIENT_REREGISTER:
			priv->counters[XVE_CLIENT_REREGISTER_COUNTER]++;
			/* As EDR uvnic needs a Heart beat restart Multicast*/
			if (!priv->vnic_type)
				set_bit(XVE_FLAG_DONT_DETACH_MCAST,
						&priv->flags);
			xve_queue_work(priv, XVE_WQ_START_FLUSHLIGHT);
			break;
	case IB_EVENT_PORT_ERR:
			priv->counters[XVE_EVENT_PORT_ERR_COUNTER]++;
			xve_queue_work(priv, XVE_WQ_START_FLUSHNORMAL);
			break;
	case IB_EVENT_PORT_ACTIVE:
			priv->counters[XVE_EVENT_PORT_ACTIVE_COUNTER]++;
			xve_queue_work(priv, XVE_WQ_START_FLUSHNORMAL);
			break;
	case IB_EVENT_LID_CHANGE:
			priv->counters[XVE_EVENT_LID_CHANGE_COUNTER]++;
			xve_queue_work(priv, XVE_WQ_START_FLUSHNORMAL);
			break;
	case IB_EVENT_PKEY_CHANGE:
			priv->counters[XVE_EVENT_PKEY_CHANGE_COUNTER]++;
			xve_queue_work(priv, XVE_WQ_START_FLUSHHEAVY);
			break;
	default:
			priv->counters[XVE_INVALID_EVENT_COUNTER]++;
			break;
	}
}
